#include once "fbSPE.bi"
#include once "glMesh.bi"

ChDir ExePath()

' NOTE: from this 3D objects the Y axis points in Z direction
' use BodySetOrientation(body,MatrixX(-90)) to fix it

const as string MD_BOX      = "2.md"  ' 1.00 x 1.00 x 1.00
const as string MD_SPHERE   = "28.md" ' 0.50 x 0.50 x 0.50
const as string MD_CAPSULE  = "29.md" ' 0.25 x 0.25 x 0.50
const as string MD_CAPSULE2 = "31.md" ' 0.75 x 0.75 x 2.00
const as string MD_DISK     = "18.md" ' 1.00 x 1.00 x 0.10

const as string MD_HEX      = "11.md" ' 0.25 x 0.25 x 0.5
const as string MD_DODE     = "16.md" ' 0.50 x 0.50 x 0.50
const as string MD_OVAL     = "20.md" ' 0.50 x 1.00 x 2.00

const as string MD_ROUND    = "14.md" ' 40.0 x 40.0 x 1.00
const as string MD_RING     = "21.md" ' ?.?? x ?.?? x ?.??

const as string MD_BARREL   = "19.md" ' 0.50 x 0.50 x 0.50

const as string MD_BARREL2  = "24.md" ' ?.?? x ?.?? x ?.??
const as string MD_POOL     = "25.md" ' 1.25 x 0.50 x 1.25
const as string MD_RAMP     = "26.md" ' 0.50 x 2.00 x 0.50

const as string MD_CORNER   = "15.md" ' ?.?? x ?.?? x ?.??
const as string MD_CORNER2  = "27.md" ' ?.?? x ?.?? x ?.??

const as string MD_STAIRS   = "12.md" ' 10.0 x 10.0 x 0.50
const as string MD_STAIRS2  = "13.md"

const as string MD_STARX    = "22.md" ' ?.?? x ?.?? x ?.??
const as string MD_BONE     = "23.md" ' 0.50 x 1.50 x 0.50
const as string MD_CAPS     = "32.md" ' 1.00 x 1.00 x 3.00


const as integer NBODIES = 1

' -1.0 ... +1.0
function rnd2 as single
  return rnd-rnd
end function

dim as SPERIGIDBODY ptr bodies(NBODIES)
dim as GLMesh ground,pool,sphere


InitOpenGL(640,480)

' create world
dim as SPEWORLD ptr world = CreateWorld()
WorldSetGravity(world,SPEVECTOR(0,-9.81,0))

' create a shape object
dim as SPESHAPE ptr shape = WorldCreateShape(world)

' create a static box for the ground in bodies(0)
ground.load MD_BOX,1000,100,1000
ShapeInitializeForStatic(shape,ground.pVertex,sizeof(SPEVERTEX),ground.pIndex,ground.nIndex\3)
bodies(0) = WorldAddRigidBody(world,shape)
BodySetPosition(bodies(0),SPEVECTOR(0,-49,0))

' create a falling sphere in bodies(1)
sphere.load MD_SPHERE ',2,2,2
ShapeInitializeAsSphere(shape,0.25)
bodies(1) = WorldAddRigidBody(world,shape)
BodySetPosition(bodies(1),SPEVECTOR(0,25,0))


' create a static pool for fluid collision in bodies(2)
pool.load MD_RAMP ,1,1,0.5
ShapeInitializeForStatic(shape,pool.pVertex,sizeof(SPEVERTEX),pool.pIndex,pool.nIndex\3)
bodies(2) = WorldAddRigidBody(world,shape)
BodySetPosition(bodies(2),SPEVECTOR(0,2,0))
BodySetOrientation(bodies(2),MatrixXYZ(-95,180,0))


' add fluid to world
dim as SPEFLUID ptr pFluid = WorldAddFluid(world)
FluidSetMaterial(pFluid,SPE_WATER) ' set water or SPE_SMOKE
FluidSetMeshGeneration(pFluid,1)          ' generate surface mesh for rendering
' add paticle to the fluid
for y as SPEFLOAT = 5 to 20 step 0.1
  for i as integer = 1 to 10
    FluidAddParticle(pFluid,SPEVECTOR(rnd2*0.25,y,rnd2*0.25))
  next
next


' free the shape object (optional)
WorldReleaseShape(world,shape)

' simple simualtion and render loop
Dim e As FB.EVENT
do
  If (ScreenEvent(@e)) Then
    Select Case e.type
    Case FB.EVENT_KEY_PRESS
      If (e.scancode = FB.SC_ESCAPE) Then exit do
    Case FB.EVENT_WINDOW_CLOSE : exit do
    end select
  End If
  ' one step in simulation
  WorldDirectStep(world)
  
  BeginFrame ' clear scene draw sky, ground with fog
  
  'ground.render BodyGetTransform(bodies(0))
  sphere.render BodyGetTransform(bodies(1))
  pool.render   BodyGetTransform(bodies(2))
  
  ' draw fluid
  FluidRender(pFluid)
  EndFrame ' flip the OpenGL context

  sleep 10
loop

ground.Release

ReleaseWorld(world)





